@@ -19,112 +19,50 @@
 #include <string.h>
 
 #include "mm-port.h"
-#include "mm-log.h"
+#include "mm-port-enums-types.h"
+#include "mm-log-object.h"
 
-G_DEFINE_TYPE (MMPort, mm_port, G_TYPE_OBJECT)
+static void log_object_iface_init (MMLogObjectInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (MMPort, mm_port, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_LOG_OBJECT, log_object_iface_init))
 
 enum {
     PROP_0,
     PROP_DEVICE,
     PROP_SUBSYS,
     PROP_TYPE,
-    PROP_CARRIER_DETECT,
     PROP_CONNECTED,
-
+    PROP_KERNEL_DEVICE,
     LAST_PROP
 };
 
-#define MM_PORT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), MM_TYPE_PORT, MMPortPrivate))
 
-typedef struct {
-    char *device;
+enum {
+    TIMED_OUT,
+    REMOVED,
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
+struct _MMPortPrivate {
+    gchar *device;
     MMPortSubsys subsys;
     MMPortType ptype;
-    gboolean carrier_detect;
     gboolean connected;
-} MMPortPrivate;
+    MMKernelDevice *kernel_device;
+};
 
 /*****************************************************************************/
 
-const char *
-mm_port_subsys_to_name (MMPortSubsys psubsys)
-{
-    switch (psubsys) {
-    case MM_PORT_SUBSYS_TTY:
-        return "tty";
-    case MM_PORT_SUBSYS_NET:
-        return "net";
-    default:
-        break;
-    }
-    return "(unknown)";
-}
-
-const char *
-mm_port_type_to_name (MMPortType ptype)
-{
-    switch (ptype) {
-    case MM_PORT_TYPE_PRIMARY:
-        return "primary";
-    case MM_PORT_TYPE_SECONDARY:
-        return "secondary";
-    case MM_PORT_TYPE_IGNORED:
-        return "ignored";
-    case MM_PORT_TYPE_QCDM:
-        return "QCDM";
-    default:
-        break;
-    }
-    return "(unknown)";
-}
-
-/*****************************************************************************/
-
-static GObject*
-constructor (GType type,
-             guint n_construct_params,
-             GObjectConstructParam *construct_params)
-{
-    GObject *object;
-    MMPortPrivate *priv;
-
-    object = G_OBJECT_CLASS (mm_port_parent_class)->constructor (type,
-                                                                 n_construct_params,
-                                                                 construct_params);
-    if (!object)
-        return NULL;
-
-    priv = MM_PORT_GET_PRIVATE (object);
-
-    if (!priv->device) {
-        g_warning ("MMPort: no device provided");
-        g_object_unref (object);
-        return NULL;
-    }
-
-    if (priv->subsys == MM_PORT_SUBSYS_UNKNOWN) {
-        g_warning ("MMPort: invalid port subsystem");
-        g_object_unref (object);
-        return NULL;
-    }
-
-    if (priv->ptype == MM_PORT_TYPE_UNKNOWN) {
-        g_warning ("MMPort: invalid port type");
-        g_object_unref (object);
-        return NULL;
-    }
-
-    return object;
-}
-
-
 const char *
 mm_port_get_device (MMPort *self)
 {
     g_return_val_if_fail (self != NULL, NULL);
     g_return_val_if_fail (MM_IS_PORT (self), NULL);
 
-    return MM_PORT_GET_PRIVATE (self)->device;
+    return self->priv->device;
 }
 
 MMPortSubsys
@@ -133,7 +71,7 @@ mm_port_get_subsys (MMPort *self)
     g_return_val_if_fail (self != NULL, MM_PORT_SUBSYS_UNKNOWN);
     g_return_val_if_fail (MM_IS_PORT (self), MM_PORT_SUBSYS_UNKNOWN);
 
-    return MM_PORT_GET_PRIVATE (self)->subsys;
+    return self->priv->subsys;
 }
 
 MMPortType
@@ -142,16 +80,7 @@ mm_port_get_port_type (MMPort *self)
     g_return_val_if_fail (self != NULL, MM_PORT_TYPE_UNKNOWN);
     g_return_val_if_fail (MM_IS_PORT (self), MM_PORT_TYPE_UNKNOWN);
 
-    return MM_PORT_GET_PRIVATE (self)->ptype;
-}
-
-gboolean
-mm_port_get_carrier_detect (MMPort *self)
-{
-    g_return_val_if_fail (self != NULL, MM_PORT_TYPE_UNKNOWN);
-    g_return_val_if_fail (MM_IS_PORT (self), MM_PORT_TYPE_UNKNOWN);
-
-    return MM_PORT_GET_PRIVATE (self)->carrier_detect;
+    return self->priv->ptype;
 }
 
 gboolean
@@ -160,59 +89,79 @@ mm_port_get_connected (MMPort *self)
     g_return_val_if_fail (self != NULL, FALSE);
     g_return_val_if_fail (MM_IS_PORT (self), FALSE);
 
-    return MM_PORT_GET_PRIVATE (self)->connected;
+    return self->priv->connected;
 }
 
 void
 mm_port_set_connected (MMPort *self, gboolean connected)
 {
-    MMPortPrivate *priv;
-
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_PORT (self));
 
-    priv = MM_PORT_GET_PRIVATE (self);
-    if (priv->connected != connected) {
-        priv->connected = connected;
+    if (self->priv->connected != connected) {
+        self->priv->connected = connected;
         g_object_notify (G_OBJECT (self), MM_PORT_CONNECTED);
-
-        mm_dbg ("(%s): port now %s",
-                priv->device,
-                connected ? "connected" : "disconnected");
+        mm_obj_dbg (self, "port now %s", connected ? "connected" : "disconnected");
     }
 }
 
+MMKernelDevice *
+mm_port_peek_kernel_device (MMPort *self)
+{
+    g_return_val_if_fail (MM_IS_PORT (self), NULL);
+
+    return self->priv->kernel_device;
+}
+
+/*****************************************************************************/
+
+static gchar *
+log_object_build_id (MMLogObject *_self)
+{
+    MMPort *self;
+
+    self = MM_PORT (_self);
+    return g_strdup_printf ("%s/%s",
+                            mm_port_get_device (self),
+                            mm_port_type_get_string (mm_port_get_port_type (self)));
+}
+
 /*****************************************************************************/
 
 static void
 mm_port_init (MMPort *self)
 {
+    self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, MM_TYPE_PORT, MMPortPrivate);
 }
 
 static void
-set_property (GObject *object, guint prop_id,
-              const GValue *value, GParamSpec *pspec)
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
 {
-    MMPortPrivate *priv = MM_PORT_GET_PRIVATE (object);
+    MMPort *self = MM_PORT (object);
 
     switch (prop_id) {
     case PROP_DEVICE:
         /* Construct only */
-        priv->device = g_value_dup_string (value);
+        self->priv->device = g_value_dup_string (value);
         break;
     case PROP_SUBSYS:
         /* Construct only */
-        priv->subsys = g_value_get_uint (value);
+        self->priv->subsys = g_value_get_uint (value);
         break;
     case PROP_TYPE:
         /* Construct only */
-        priv->ptype = g_value_get_uint (value);
-        break;
-    case PROP_CARRIER_DETECT:
-        priv->carrier_detect = g_value_get_boolean (value);
+        self->priv->ptype = g_value_get_uint (value);
         break;
     case PROP_CONNECTED:
-        priv->connected = g_value_get_boolean (value);
+        self->priv->connected = g_value_get_boolean (value);
+        break;
+    case PROP_KERNEL_DEVICE:
+        /* Not construct only, but only set once */
+        g_assert (!self->priv->kernel_device);
+        self->priv->kernel_device = g_value_dup_object (value);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -224,23 +173,23 @@ static void
 get_property (GObject *object, guint prop_id,
               GValue *value, GParamSpec *pspec)
 {
-    MMPortPrivate *priv = MM_PORT_GET_PRIVATE (object);
+    MMPort *self = MM_PORT (object);
 
     switch (prop_id) {
     case PROP_DEVICE:
-        g_value_set_string (value, priv->device);
+        g_value_set_string (value, self->priv->device);
         break;
     case PROP_SUBSYS:
-        g_value_set_uint (value, priv->subsys);
+        g_value_set_uint (value, self->priv->subsys);
         break;
     case PROP_TYPE:
-        g_value_set_uint (value, priv->ptype);
-        break;
-    case PROP_CARRIER_DETECT:
-        g_value_set_boolean (value, priv->carrier_detect);
+        g_value_set_uint (value, self->priv->ptype);
         break;
     case PROP_CONNECTED:
-        g_value_set_boolean (value, priv->connected);
+        g_value_set_boolean (value, self->priv->connected);
+        break;
+    case PROP_KERNEL_DEVICE:
+        g_value_set_object (value, self->priv->kernel_device);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -251,13 +200,29 @@ get_property (GObject *object, guint prop_id,
 static void
 finalize (GObject *object)
 {
-    MMPortPrivate *priv = MM_PORT_GET_PRIVATE (object);
+    MMPort *self = MM_PORT (object);
 
-    g_free (priv->device);
+    g_free (self->priv->device);
 
     G_OBJECT_CLASS (mm_port_parent_class)->finalize (object);
 }
 
+static void
+dispose (GObject *object)
+{
+    MMPort *self = MM_PORT (object);
+
+    g_clear_object (&self->priv->kernel_device);
+
+    G_OBJECT_CLASS (mm_port_parent_class)->dispose (object);
+}
+
+static void
+log_object_iface_init (MMLogObjectInterface *iface)
+{
+    iface->build_id = log_object_build_id;
+}
+
 static void
 mm_port_class_init (MMPortClass *klass)
 {
@@ -266,10 +231,10 @@ mm_port_class_init (MMPortClass *klass)
     g_type_class_add_private (object_class, sizeof (MMPortPrivate));
 
     /* Virtual methods */
-    object_class->constructor = constructor;
     object_class->set_property = set_property;
     object_class->get_property = get_property;
     object_class->finalize = finalize;
+    object_class->dispose = dispose;
 
     g_object_class_install_property
         (object_class, PROP_DEVICE,
@@ -299,14 +264,6 @@ mm_port_class_init (MMPortClass *klass)
                             MM_PORT_TYPE_UNKNOWN,
                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-    g_object_class_install_property
-        (object_class, PROP_CARRIER_DETECT,
-         g_param_spec_boolean (MM_PORT_CARRIER_DETECT,
-                               "Carrier Detect",
-                               "Has Carrier Detect",
-                               TRUE,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
     g_object_class_install_property
         (object_class, PROP_CONNECTED,
          g_param_spec_boolean (MM_PORT_CONNECTED,
@@ -314,4 +271,30 @@ mm_port_class_init (MMPortClass *klass)
                                "Is connected for data and not usable for control",
                                FALSE,
                                G_PARAM_READWRITE));
+
+    g_object_class_install_property
+        (object_class, PROP_KERNEL_DEVICE,
+         g_param_spec_object (MM_PORT_KERNEL_DEVICE,
+                              "Kernel device",
+                              "kernel device object",
+                              MM_TYPE_KERNEL_DEVICE,
+                              G_PARAM_READWRITE));
+
+    signals[TIMED_OUT] =
+        g_signal_new (MM_PORT_SIGNAL_TIMED_OUT,
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (MMPortClass, timed_out),
+                      NULL, NULL,
+                      g_cclosure_marshal_generic,
+                      G_TYPE_NONE, 1, G_TYPE_UINT);
+
+    signals[REMOVED] =
+        g_signal_new (MM_PORT_SIGNAL_REMOVED,
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (MMPortClass, removed),
+                      NULL, NULL,
+                      g_cclosure_marshal_generic,
+                      G_TYPE_NONE, 0);
 }
